@@ -9,8 +9,10 @@ __all__ = ["TemporalDictionaryEnsemble", "IndividualTDE", "histogram_intersectio
 
 import math
 import time
+from collections import defaultdict
 
 import numpy as np
+from joblib import Parallel, delayed
 from sklearn.gaussian_process import GaussianProcessRegressor
 from sklearn.utils import check_random_state
 from sklearn.utils.multiclass import class_distribution
@@ -25,7 +27,7 @@ class TemporalDictionaryEnsemble(BaseClassifier):
     """
     Temporal Dictionary Ensemble (TDE) as described in [1].
 
-    Overview: Input n series length m
+    Overview: Input n series length m with d dimensions
     TDE searches k parameter values selected using a Gaussian processes
     regressor, evaluating each with a LOOCV. It then retains s
     ensemble members.
@@ -39,12 +41,13 @@ class TemporalDictionaryEnsemble(BaseClassifier):
     for any combination, an individual TDE classifier slides a window of
     length w along the series. The w length window is shortened to
     an l length word through taking a Fourier transform and keeping the
-    first l/2 complex coefficients. These l
-    coefficients are then discretised into alpha possible values, to form a
-    word length l using breakpoints found using b. A histogram of words for
-    each series is formed and stored, using a spatial pyramid of h levels.
-    fit involves finding n histograms.
+    first l/2 complex coefficients. These lcoefficients are then discretised
+    into alpha possible values, to form a word length l using breakpoints
+    found using b. A histogram of words for each series is formed and stored,
+    using a spatial pyramid of h levels. For multivariate series, accuracy
+    from a reduced histogram is used to select dimensions.
 
+    fit involves finding n histograms.
     predict uses 1 nearest neighbour with a the histogram intersection
     distance function.
 
@@ -55,15 +58,26 @@ class TemporalDictionaryEnsemble(BaseClassifier):
 
     Parameters
     ----------
-    n_parameter_samples     : int, if search is randomised, number of
-    parameter combos to try (default=250)
-    max_ensemble_size       : int or None, retain a maximum number of
-    classifiers, even if within threshold, optional (default = 100)
-    time_limit              : time contract to limit build time in minutes
-    (default=0, no limit)
-    max_win_len_prop        : maximum window length as a proportion of
-    series length (default =1)
-    min_window              : minimum window size, (default=10)
+    n_parameter_samples     : int, number of parameter combos to try
+    (default=250)
+    max_ensemble_size       : int, maximum number of classifiers
+    (default=50)
+    time_limit              : int, time contract to limit build time in
+    minutes (default=0, no limit)
+    max_win_len_prop        : float between 0 and 1, maximum window length
+    as a proportion of series length (default=1)
+    min_window              : int, minimum window size (default=10)
+    randomly_selected_params: int, number of parameters randomly selected
+    before GP is used (default=50)
+    bigrams                 : boolean or None, whether to use bigrams
+    (default=None, true for univariate, false for multivariate)
+    dim_threshold           : float between 0 and 1, dimension accuracy
+    threshold for multivariate (default=0.85)
+    max_dims                : int, max number of dimensions for multivariate
+    (default=20)
+    n_jobs                  : int, optional (default=1)
+    The number of jobs to run in parallel for both `fit` and `predict`.
+    ``-1`` means using all processors.
     random_state            : int or None, seed for random, integer,
     optional (default to no seed)
 
@@ -71,28 +85,38 @@ class TemporalDictionaryEnsemble(BaseClassifier):
     ----------
     n_classes               : extracted from the data
     n_instances             : extracted from the data
-    n_estimators           : The final number of classifiers used (
-    <=max_ensemble_size)
+    n_dims                  : extracted from the data
+    n_estimators            : The final number of classifiers used
+    (<=max_ensemble_size)
     series_length           : length of all series (assumed equal)
-    classifiers             : array of DecisionTree classifiers
+    classifiers             : array of IndividualTDE classifiers
     weights                 : weight of each classifier in the ensemble
+    weight_sum              : sum of all weights
     prev_parameters_x       : parameter value of previous classifiers for GP
     prev_parameters_y       : accuracy of previous classifiers for GP
 
     Notes
     -----
     ..[1] Matthew Middlehurst, James Large, Gavin Cawley and Anthony Bagnall
-        "The Temporal Dictionary Ensemble {(TDE)} Classifier for Time Series
+        "The Temporal Dictionary Ensemble (TDE) Classifier for Time Series
         Classification",
             in proceedings of the European Conference on Machine Learning and
             Principles and Practice of Knowledge Discovery in Databases, 2020
-    https://ueaeprints.uea.ac.uk/id/eprint/75490/
+        https://ueaeprints.uea.ac.uk/id/eprint/75490/
+
     Java version
     https://github.com/uea-machine-learning/tsml/blob/master/src/main/java/
     tsml/classifiers/dictionary_based/TDE.java
 
     """
 
+    # Capabilities: data types this classifier can handle
+    capabilities = {
+        "multivariate": True,
+        "unequal_length": False,
+        "missing_values": False,
+    }
+
     def __init__(
         self,
         n_parameter_samples=250,
@@ -101,6 +125,10 @@ class TemporalDictionaryEnsemble(BaseClassifier):
         max_win_len_prop=1,
         min_window=10,
         randomly_selected_params=50,
+        bigrams=None,
+        dim_threshold=0.85,
+        max_dims=20,
+        n_jobs=1,
         random_state=None,
     ):
         self.n_parameter_samples = n_parameter_samples
@@ -108,8 +136,15 @@ class TemporalDictionaryEnsemble(BaseClassifier):
         self.max_win_len_prop = max_win_len_prop
         self.time_limit = time_limit
         self.randomly_selected_params = randomly_selected_params
+        self.bigrams = bigrams
+
+        self.n_jobs = n_jobs
         self.random_state = random_state
 
+        # multivariate
+        self.dim_threshold = dim_threshold
+        self.max_dims = max_dims
+
         self.classifiers = []
         self.weights = []
         self.weight_sum = 0
@@ -118,6 +153,7 @@ class TemporalDictionaryEnsemble(BaseClassifier):
         self.class_dictionary = {}
         self.n_estimators = 0
         self.series_length = 0
+        self.n_dims = 0
         self.n_instances = 0
         self.prev_parameters_x = []
         self.prev_parameters_y = []
@@ -146,10 +182,10 @@ class TemporalDictionaryEnsemble(BaseClassifier):
         -------
         self : object
         """
-        X, y = check_X_y(X, y, enforce_univariate=True, coerce_to_numpy=True)
+        X, y = check_X_y(X, y, coerce_to_numpy=True)
 
         self.time_limit = self.time_limit * 60
-        self.n_instances, self.series_length = X.shape[0], X.shape[-1]
+        self.n_instances, self.n_dims, self.series_length = X.shape
         self.n_classes = np.unique(y).shape[0]
         self.classes_ = class_distribution(np.asarray(y).reshape(-1, 1))[0][0]
         for index, classVal in enumerate(self.classes_):
@@ -177,9 +213,27 @@ class TemporalDictionaryEnsemble(BaseClassifier):
 
         if self.time_limit > 0:
             self.n_parameter_samples = 0
-
+        if self.min_window > max_window + 1:
+            raise ValueError(
+                f"Error in TemporalDictionaryEnsemble, min_window ="
+                f"{self.min_window} is bigger"
+                f" than max_window ={self.max_window},"
+                f" series length is {self.series_length}"
+                f" try set min_window to be smaller than series length in "
+                f"the constructor, but the classifier may not work at "
+                f"all with very short series"
+            )
         rng = check_random_state(self.random_state)
 
+        if self.bigrams is None:
+            if self.n_dims > 1:
+                use_bigrams = False
+            else:
+                use_bigrams = True
+        else:
+            use_bigrams = self.bigrams
+
+        # use time limit or n_parameter_samples if limit is 0
         while (
             train_time < self.time_limit or num_classifiers < self.n_parameter_samples
         ) and len(possible_parameters) > 0:
@@ -202,9 +256,13 @@ class TemporalDictionaryEnsemble(BaseClassifier):
             tde = IndividualTDE(
                 *parameters,
                 alphabet_size=self.alphabet_size,
-                random_state=self.random_state
+                bigrams=use_bigrams,
+                dim_threshold=self.dim_threshold,
+                max_dims=self.max_dims,
+                random_state=self.random_state,
             )
             tde.fit(X_subsample, y_subsample)
+            tde.subsample = subsample
 
             tde.accuracy = self._individual_train_acc(
                 tde, y_subsample, subsample_size, lowest_acc
@@ -245,7 +303,7 @@ class TemporalDictionaryEnsemble(BaseClassifier):
 
     def predict_proba(self, X):
         self.check_is_fitted()
-        X = check_X(X, enforce_univariate=True, coerce_to_numpy=True)
+        X = check_X(X, coerce_to_numpy=True)
 
         sums = np.zeros((X.shape[0], self.n_classes))
 
@@ -269,24 +327,6 @@ class TemporalDictionaryEnsemble(BaseClassifier):
 
         return min_acc, min_acc_idx
 
-    def _get_train_probs(self, X):
-        num_inst = X.shape[0]
-        results = np.zeros((num_inst, self.n_classes))
-        divisor = np.ones(self.n_classes) * np.sum(self.weights)
-        for i in range(num_inst):
-            sums = np.zeros(self.n_classes)
-
-            for n, clf in enumerate(self.classifiers):
-                sums[
-                    self.class_dictionary.get(clf._train_predict(i), -1)
-                ] += self.weights[n]
-
-            dists = sums / divisor
-            for n in range(self.n_classes):
-                results[i][n] = dists[n]
-
-        return results
-
     def _unique_parameters(self, max_window, win_inc):
         possible_parameters = [
             [win_size, word_len, normalise, levels, igb]
@@ -299,18 +339,64 @@ class TemporalDictionaryEnsemble(BaseClassifier):
 
         return possible_parameters
 
+    def _get_train_probs(self, X):
+        num_inst = X.shape[0]
+        results = np.zeros((num_inst, self.n_classes))
+        for i in range(num_inst):
+            divisor = 0
+            sums = np.zeros(self.n_classes)
+
+            cls_idx = []
+            for n, clf in enumerate(self.classifiers):
+                idx = np.where(clf.subsample == i)
+                if len(idx[0]) > 0:
+                    cls_idx.append([n, idx[0][0]])
+
+            preds = Parallel(n_jobs=self.n_jobs)(
+                delayed(self.classifiers[cls[0]]._train_predict)(
+                    cls[1],
+                )
+                for cls in cls_idx
+            )
+
+            for n, pred in enumerate(preds):
+                sums[self.class_dictionary.get(pred, -1)] += self.weights[cls_idx[n][0]]
+                divisor += self.weights[cls_idx[n][0]]
+
+            results[i] = (
+                np.ones(self.n_classes) * (1 / self.n_classes)
+                if divisor == 0
+                else sums / (np.ones(self.n_classes) * divisor)
+            )
+
+        return results
+
     def _individual_train_acc(self, tde, y, train_size, lowest_acc):
         correct = 0
         required_correct = int(lowest_acc * train_size)
 
-        for i in range(train_size):
-            if correct + train_size - i < required_correct:
-                return -1
+        if self.n_jobs > 1:
+            c = Parallel(n_jobs=self.n_jobs)(
+                delayed(tde._train_predict)(
+                    i,
+                )
+                for i in range(train_size)
+            )
 
-            c = tde._train_predict(i)
+            for i in range(train_size):
+                if correct + train_size - i < required_correct:
+                    return -1
+                elif c[i] == y[i]:
+                    correct += 1
+        else:
+            for i in range(train_size):
+                if correct + train_size - i < required_correct:
+                    return -1
 
-            if c == y[i]:
-                correct += 1
+                c = tde._train_predict(i)
+
+                if c == y[i]:
+                    correct += 1
 
         return correct / train_size
 
@@ -326,6 +412,10 @@ class IndividualTDE(BaseClassifier):
         levels=1,
         igb=False,
         alphabet_size=4,
+        bigrams=True,
+        dim_threshold=0.85,
+        max_dims=20,
+        n_jobs=1,
         random_state=None,
     ):
         self.window_size = window_size
@@ -334,25 +424,25 @@ class IndividualTDE(BaseClassifier):
         self.levels = levels
         self.igb = igb
         self.alphabet_size = alphabet_size
+        self.bigrams = bigrams
 
+        # multivariate
+        self.dim_threshold = dim_threshold
+        self.max_dims = max_dims
+
+        self.n_jobs = n_jobs
         self.random_state = random_state
 
-        binning_method = "information-gain" if igb else "equi-depth"
-
-        self.transformer = SFA(
-            word_length=word_length,
-            alphabet_size=alphabet_size,
-            window_size=window_size,
-            norm=norm,
-            levels=levels,
-            binning_method=binning_method,
-            bigrams=True,
-            remove_repeat_words=True,
-            save_words=False,
-        )
+        self.transformers = []
         self.transformed_data = []
         self.accuracy = 0
+        self.subsample = []
 
+        self.n_instances = 0
+        self.n_dims = 0
+        self.series_length = 0
+        self.highest_dim_bit = 0
+        self.dims = []
         self.class_vals = []
         self.num_classes = 0
         self.classes_ = []
@@ -360,42 +450,80 @@ class IndividualTDE(BaseClassifier):
         super(IndividualTDE, self).__init__()
 
     def fit(self, X, y):
-        X, y = check_X_y(X, y, enforce_univariate=True, coerce_to_numpy=True)
-
-        sfa = self.transformer.fit_transform(X, y)
-        self.transformed_data = sfa[0]  # .iloc[:, 0]
+        X, y = check_X_y(X, y, coerce_to_numpy=True)
 
+        self.n_instances, self.n_dims, self.series_length = X.shape
         self.class_vals = y
         self.num_classes = np.unique(y).shape[0]
         self.classes_ = class_distribution(np.asarray(y).reshape(-1, 1))[0][0]
         for index, classVal in enumerate(self.classes_):
             self.class_dictionary[classVal] = index
 
+        # select dimensions using accuracy estimate if multivariate
+        if self.n_dims > 1:
+            self.dims, self.transformers = self._select_dims(X, y)
+
+            words = [defaultdict(int) for _ in range(self.n_instances)]
+
+            for i, dim in enumerate(self.dims):
+                X_dim = X[:, dim, :].reshape(self.n_instances, 1, self.series_length)
+                dim_words = self.transformers[i].transform(X_dim, y)
+                dim_words = dim_words[0]
+
+                for i in range(self.n_instances):
+                    for word, count in dim_words[i].items():
+                        words[i][word << self.highest_dim_bit | dim] = count
+
+            self.transformed_data = words
+        else:
+            self.transformers.append(
+                SFA(
+                    word_length=self.word_length,
+                    alphabet_size=self.alphabet_size,
+                    window_size=self.window_size,
+                    norm=self.norm,
+                    levels=self.levels,
+                    binning_method="information-gain" if self.igb else "equi-depth",
+                    bigrams=self.bigrams,
+                    remove_repeat_words=True,
+                    save_words=False,
+                    n_jobs=self.n_jobs,
+                )
+            )
+            sfa = self.transformers[0].fit_transform(X, y)
+            self.transformed_data = sfa[0]
+
         self._is_fitted = True
         return self
 
     def predict(self, X):
         self.check_is_fitted()
-        X = check_X(X, enforce_univariate=True, coerce_to_numpy=True)
+        X = check_X(X, coerce_to_numpy=True)
+        num_cases = X.shape[0]
 
-        rng = check_random_state(self.random_state)
+        if self.n_dims > 1:
+            words = [defaultdict(int) for _ in range(num_cases)]
 
-        classes = []
-        test_bags = self.transformer.transform(X)
-        test_bags = test_bags[0]  # .iloc[:, 0]
+            for i, dim in enumerate(self.dims):
+                X_dim = X[:, dim, :].reshape(num_cases, 1, self.series_length)
+                dim_words = self.transformers[i].transform(X_dim)
+                dim_words = dim_words[0]
 
-        for test_bag in test_bags:
-            best_sim = -1
-            nn = None
+                for i in range(num_cases):
+                    for word, count in dim_words[i].items():
+                        words[i][word << self.highest_dim_bit | dim] = count
 
-            for n, bag in enumerate(self.transformed_data):
-                sim = histogram_intersection(test_bag, bag)
+            test_bags = words
+        else:
+            test_bags = self.transformers[0].transform(X)
+            test_bags = test_bags[0]
 
-                if sim > best_sim or (sim == best_sim and rng.random() < 0.5):
-                    best_sim = sim
-                    nn = self.class_vals[n]
-
-            classes.append(nn)
+        classes = Parallel(n_jobs=self.n_jobs)(
+            delayed(self._test_nn)(
+                test_bag,
+            )
+            for test_bag in test_bags
+        )
 
         return np.array(classes)
 
@@ -408,12 +536,91 @@ class IndividualTDE(BaseClassifier):
 
         return dists
 
-    def _train_predict(self, train_num):
-        test_bag = self.transformed_data[train_num]
+    def _test_nn(self, test_bag):
+        rng = check_random_state(self.random_state)
+
         best_sim = -1
         nn = None
 
         for n, bag in enumerate(self.transformed_data):
+            sim = histogram_intersection(test_bag, bag)
+
+            if sim > best_sim or (sim == best_sim and rng.random() < 0.5):
+                best_sim = sim
+                nn = self.class_vals[n]
+
+        return nn
+
+    def _select_dims(self, X, y):
+        self.highest_dim_bit = (math.ceil(math.log2(self.n_dims))) + 1
+        accs = []
+        transformers = []
+
+        # select dimensions based on reduced bag size accuracy
+        for i in range(self.n_dims):
+            self.dims.append(i)
+            transformers.append(
+                SFA(
+                    word_length=self.word_length,
+                    alphabet_size=self.alphabet_size,
+                    window_size=self.window_size,
+                    norm=self.norm,
+                    levels=self.levels,
+                    binning_method="information-gain" if self.igb else "equi-depth",
+                    bigrams=self.bigrams,
+                    remove_repeat_words=True,
+                    save_words=False,
+                    save_binning_dft=True,
+                    n_jobs=self.n_jobs,
+                )
+            )
+
+            X_dim = X[:, i, :].reshape(self.n_instances, 1, self.series_length)
+
+            transformers[i].fit(X_dim, y)
+            sfa = transformers[i].transform(
+                X_dim,
+                y,
+                transformers[i].binning_dft,
+            )
+            transformers[i].binning_dft = None
+
+            correct = 0
+            for i in range(self.n_instances):
+                if self._train_predict(i, sfa[0]) == y[i]:
+                    correct = correct + 1
+
+            accs.append(correct)
+
+        max_acc = max(accs)
+
+        dims = []
+        fin_transformers = []
+        for i in range(self.n_dims):
+            if accs[i] >= max_acc * self.dim_threshold:
+                dims.append(i)
+                fin_transformers.append(transformers[i])
+
+        if len(dims) > self.max_dims:
+            idx = self.random_state.choice(
+                len(dims),
+                self.max_dims,
+                replace=False,
+            ).tolist()
+            dims = [dims[i] for i in idx]
+            fin_transformers = [fin_transformers[i] for i in idx]
+
+        return dims, fin_transformers
+
+    def _train_predict(self, train_num, bags=None):
+        if bags is None:
+            bags = self.transformed_data
+
+        test_bag = bags[train_num]
+        best_sim = -1
+        nn = None
+
+        for n, bag in enumerate(bags):
             if n == train_num:
                 continue
 
