@@ -34,7 +34,9 @@ from sktime.forecasting.tests._config import TEST_OOS_FHS
 from sktime.forecasting.tests._config import TEST_STEP_LENGTHS
 from sktime.forecasting.tests._config import TEST_WINDOW_LENGTHS
 from sktime.forecasting.tests._config import VALID_INDEX_FH_COMBINATIONS
-from sktime.performance_metrics.forecasting import smape_loss
+from sktime.performance_metrics.forecasting import (
+    mean_absolute_percentage_error,
+)
 from sktime.utils import all_estimators
 from sktime.utils._testing.estimator_checks import _construct_instance
 from sktime.utils._testing.forecasting import _assert_correct_pred_time_index
@@ -233,7 +235,9 @@ def test_score(Forecaster, fh):
     y_pred = f.predict()
 
     fh_idx = check_fh(fh).to_indexer()  # get zero based index
-    expected = smape_loss(y_pred, y_test.iloc[fh_idx])
+    expected = mean_absolute_percentage_error(
+        y_pred, y_test.iloc[fh_idx], symmetric=True
+    )
 
     # compare with actual score
     f = _construct_instance(Forecaster)
