@@ -34,18 +34,19 @@ def check_series(
 
     Parameters
     ----------
-    Z : pd.Series, pd.DataFrame
-        Univariate or multivariate time series
-    enforce_univariate : bool, optional (default=False)
+    Z : pd.Series, pd.DataFrame, np.ndarray
+        Univariate or multivariate time series. NumPy ndarray only allowed
+        if `allow_numpy` = True.
+    enforce_univariate : bool, default = False
         If True, multivariate Z will raise an error.
-    allow_empty : bool
-        If False, empty Z will raise an error
-    enforce_index_type : type, optional (default=None)
+    allow_empty : bool, default = False
+    allow_numpy : bool, default = True
+    enforce_index_type : type, default = None
         type of time index
 
     Returns
     -------
-    Z : pd.Series, pd.DataFrame
+    Z : pd.Series, pd.DataFrame, np.ndarray
         Validated time series
 
     Raises
@@ -69,10 +70,12 @@ def check_series(
     if enforce_univariate:
         _check_is_univariate(Z)
 
-    # check time index
-    check_time_index(
-        Z.index, allow_empty=allow_empty, enforce_index_type=enforce_index_type
-    )
+    # check time index if input data is not an NumPy ndarray
+    if not isinstance(Z, np.ndarray):
+        check_time_index(
+            Z.index, allow_empty=allow_empty, enforce_index_type=enforce_index_type
+        )
+
     return Z
 
 
@@ -132,7 +135,7 @@ def check_equal_time_index(*ys):
 
     Parameters
     ----------
-    ys : pd.Series or pd.DataFrame
+    ys : pd.Series, pd.DataFrame or np.ndarray
         One or more time series
 
     Raises
